// based off '@graphql-codegen/typescript-graphql-files-modules'
import { basename, relative } from 'path';
import { concatAST, Kind, } from 'graphql';
import { pascalCase } from 'pascal-case';
import { TypeScriptDocumentNodesVisitor } from './visitor';
export const plugin = (schema, documents, config) => {
    const { modulePathPrefix = '', relativeTo, prefix = '', typedDocumentNodeModule, } = config;
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    const allAst = concatAST(documents.map((v) => v.document));
    const allFragments = [
        ...allAst.definitions.filter((d) => d.kind === Kind.FRAGMENT_DEFINITION).map((fragmentDef) => ({
            node: fragmentDef,
            name: fragmentDef.name.value,
            onType: fragmentDef.typeCondition.name.value,
            isExternal: false,
        })),
    ];
    const visitor = new TypeScriptDocumentNodesVisitor(schema, allFragments, config, documents);
    const useRelative = Boolean(relativeTo);
    const mappedDocuments = documents.reduce((prev, documentRecord) => {
        const fileName = useRelative
            ? // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                relative(relativeTo, documentRecord.location)
            : // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                basename(documentRecord.location);
        if (!prev[fileName]) {
            prev[fileName] = [];
        }
        if (!documentRecord.document)
            return prev;
        prev[fileName].push(...documentRecord.document.definitions.filter((document) => document.kind === 'OperationDefinition' ||
            document.kind === 'FragmentDefinition'));
        return prev;
    }, {});
    return Object.keys(mappedDocuments)
        .filter((fileName) => mappedDocuments[fileName].length > 0)
        .map((fileName) => {
        var _a;
        const operations = mappedDocuments[fileName];
        const namedOperations = operations.filter((d) => { var _a; return (_a = d.name) === null || _a === void 0 ? void 0 : _a.value; });
        const nodesWithNames = namedOperations.map((node) => {
            // based on https://github.com/dotansimha/graphql-code-generator/blob/a790904a1ae42e63ef8552c9b0145f5f3292d474/packages/plugins/other/visitor-plugin-common/src/client-side-base-visitor.ts#L552-L566
            const documentVariableName = visitor.convertName(node, {
                suffix: visitor.config.documentVariableSuffix,
                prefix: visitor.config.documentVariablePrefix,
                useTypesPrefix: false,
            });
            if (node.kind === 'FragmentDefinition') {
                return {
                    documentVariableName,
                    resultType: visitor.getFragmentName(node),
                    variablesType: 'unknown',
                    node,
                };
            }
            const operationType = pascalCase(node.operation);
            const operationTypeSuffix = visitor.getOperationSuffix(node, operationType);
            const resultType = visitor.convertName(node, {
                suffix: `${operationTypeSuffix}${visitor.config.operationResultSuffix}`,
            });
            const variablesType = visitor.convertName(node, {
                suffix: `${operationTypeSuffix}Variables`,
            });
            return {
                documentVariableName,
                resultType,
                variablesType,
                node,
            };
        });
        if (nodesWithNames.length === 0)
            return ``;
        const defaultExport = (_a = nodesWithNames.find(({ node }) => node.kind === 'OperationDefinition')) !== null && _a !== void 0 ? _a : nodesWithNames[0];
        if (!defaultExport)
            return '';
        return `
declare module '${prefix}${modulePathPrefix}${fileName}' {
  import { TypedDocumentNode as DocumentNode } from '@graphql-typed-document-node/core';
  const defaultDocument: DocumentNode<import('${typedDocumentNodeModule}').${defaultExport.resultType}, import('${typedDocumentNodeModule}').${defaultExport.variablesType}>;
  ${nodesWithNames
            .map(({ node, resultType }) => { var _a; return `export const ${(_a = node.name) === null || _a === void 0 ? void 0 : _a.value}: import('${typedDocumentNodeModule}').${resultType};`; })
            .join('\n')}
  export default defaultDocument;
}
    `;
    })
        .join('\n');
};
export const validate = (schema, documents, config, outputFile) => {
    if (!outputFile.endsWith('.d.ts')) {
        throw new Error(`Plugin "codegen-typescript-graphql-module-declarations" requires extension to be ".d.ts"!`);
    }
};
//# sourceMappingURL=index.js.map